@@ -31,7 +31,7 @@ namespace Tomboy
 		DateTime change_date;
 		DateTime metadata_change_date;
 
-		int cursor_pos;
+		int cursor_pos, selection_bound_pos;
 		int width, height;
 		int x, y;
 		bool open_on_startup;
@@ -46,6 +46,7 @@ namespace Tomboy
 			this.text = "";
 			x = noPosition;
 			y = noPosition;
+			selection_bound_pos = noPosition;
 
 			tags = new Dictionary<string, Tag> ();
 
@@ -121,7 +122,7 @@ namespace Tomboy
 		}
 		
 
-		// FIXME: the next five attributes don't belong here (the data
+		// FIXME: the next six attributes don't belong here (the data
 		// model), but belong into the view; for now they are kept here
 		// for backwards compatibility
 
@@ -134,6 +135,16 @@ namespace Tomboy
 				cursor_pos = value;
 			}
 		}
+		
+		public int SelectionBoundPosition
+		{
+			get {
+				return selection_bound_pos;
+			}
+			set {
+				selection_bound_pos = value;
+			}
+		}
 
 		public int Width
 		{
@@ -317,6 +328,13 @@ namespace Tomboy
 					cursor = buffer.GetIterAtLine (2);
 				}
 				buffer.PlaceCursor (cursor);
+				
+				if (data.SelectionBoundPosition >= 0) {
+					// Move selection bound to last-saved position
+					Gtk.TextIter selection_bound;
+					selection_bound = buffer.GetIterAtOffset (data.SelectionBoundPosition);
+					buffer.MoveMark (buffer.SelectionBound.Name, selection_bound);
+				}
 
 				// New events should create Undo actions
 				buffer.Undoer.ThawUndo ();
@@ -535,7 +553,7 @@ namespace Tomboy
 		{
 			if (NoteTagTable.TagIsSerializable (args.Tag)) {
 				DebugSave ("BufferTagApplied queueing save: {0}", args.Tag.Name);
-				QueueSave (ChangeType.ContentChanged);
+				QueueSave (TagTable.GetChangeType (args.Tag));
 			}
 		}
 
@@ -543,18 +561,20 @@ namespace Tomboy
 		{
 			if (NoteTagTable.TagIsSerializable (args.Tag)) {
 				DebugSave ("BufferTagRemoved queueing save: {0}", args.Tag.Name);
-				QueueSave (ChangeType.ContentChanged);
+				QueueSave (TagTable.GetChangeType (args.Tag));
 			}
 		}
 
-		void BufferInsertMarkSet (object sender, Gtk.MarkSetArgs args)
+		void OnBufferMarkSet (object sender, Gtk.MarkSetArgs args)
 		{
-			if (args.Mark != buffer.InsertMark)
+			if (args.Mark == buffer.InsertMark)
+				data.Data.CursorPosition = args.Location.Offset;
+			else if (args.Mark == buffer.SelectionBound)
+				data.Data.SelectionBoundPosition = args.Location.Offset;
+			else
 				return;
 
-			data.Data.CursorPosition = args.Location.Offset;
-
-			DebugSave ("BufferInsertSetMark queueing save");
+			DebugSave ("OnBufferSetMark queueing save");
 			QueueSave (ChangeType.NoChange);
 		}
 
@@ -683,7 +703,7 @@ namespace Tomboy
 		
 		public bool ContainsTag (Tag tag)
 		{
-			if (data.Data.Tags.ContainsKey (tag.NormalizedName) == true)
+			if (tag != null && data.Data.Tags.ContainsKey (tag.NormalizedName))
 				return true;
 			
 			return false;
@@ -1082,7 +1102,7 @@ namespace Tomboy
 					buffer.Changed += OnBufferChanged;
 					buffer.TagApplied += BufferTagApplied;
 					buffer.TagRemoved += BufferTagRemoved;
-					buffer.MarkSet += BufferInsertMarkSet;
+					buffer.MarkSet += OnBufferMarkSet;
 				}
 				return buffer;
 			}
@@ -1125,8 +1145,76 @@ namespace Tomboy
 						window.SetDefaultSize (data.Data.Width,
 						                       data.Data.Height);
 
-					if (data.Data.HasPosition ())
+					if (data.Data.HasPosition ()) {
+						// This would return the monitor where the point is, or the nearest one,
+						// protecting us from restoring onto non-existing monitor
+						int mon = window.Screen.GetMonitorAtPoint(data.Data.X, data.Data.Y);
+						Gdk.Rectangle mon_geom = window.Screen.GetMonitorGeometry(mon);
+						if (!mon_geom.Contains(data.Data.X, data.Data.Y)) {
+							// If we're here, then the saved position was
+							// on some now-disconnected monitor
+							Logger.Debug("Saved note position {0} x {1} is off-screen, " +
+							             "we'll move it to the center of the nearest monitor",
+							             data.Data.X, data.Data.Y);
+							data.Data.X = mon_geom.Right / 2 - data.Data.Width / 2;
+							data.Data.Y = mon_geom.Bottom / 2 - data.Data.Height / 2;
+						}
+
+						// 100px is a clearance space to accommodate taskbars and such.
+						// There's no nice and cross-platform way to get this information, but
+						// System.Windows.Forms.Screen.GetWorkingArea which adds unwanted deps
+						int clearance = 100;
+						int ne_x = data.Data.X + data.Data.Width + clearance;
+						if (ne_x > mon_geom.Right) {
+							Logger.Debug("Note runs beyond the screen on X coord, " +
+							             "we'll adjust the position");
+							// We'll first try to just move the window so that it's shown in full
+							int new_x = data.Data.X - (ne_x - mon_geom.Right);
+							if (new_x >= mon_geom.Left+clearance) {
+								// New position places the whole note within the screen
+								data.Data.X = new_x;
+								Logger.Debug("Moving note to {0} on X axis", new_x);
+							} else {
+								// Adjusted position is still beyond the allowable area,
+								// so we'll adjust & resize the note window
+								data.Data.X = mon_geom.Left + clearance;
+								data.Data.Width = mon_geom.Width - 2*clearance;
+								int win_w, win_h;
+								window.GetSize(out win_w, out win_h);
+								window.Resize(data.Data.Width, win_h);
+								Logger.Debug("Note is too wide to fully fit, " +
+								             "can't move nicely, " +
+								             "moving to fallback position {0} and resizing to {1} on X axis",
+								             data.Data.X, data.Data.Width);
+							}
+						}
+
+						int sw_y = data.Data.Y + data.Data.Height + clearance;
+						if (sw_y > mon_geom.Bottom) {
+							Logger.Debug("Note runs beyond the screen on Y coord, " +
+							             "we'll adjust the position");
+							// We'll first try to just move the window so that it's shown in full
+							int new_y = data.Data.Y - (sw_y - mon_geom.Bottom);
+							if (new_y >= mon_geom.Top+clearance) {
+								data.Data.Y = new_y;
+								Logger.Debug("Moving note to {0} on Y axis", new_y);
+							} else {
+								// Adjusted position is still beyond the allowable area,
+								// so we'll adjust & resize the note window
+								data.Data.Y = mon_geom.Top + clearance;
+								data.Data.Height = mon_geom.Height - 2*clearance;
+								int win_w, win_h;
+								window.GetSize(out win_w, out win_h);
+								window.Resize(win_w, data.Data.Height);
+								Logger.Debug("Note is too tall to fully fit, " +
+								             "can't move nicely, " +
+								             "moving to fallback position {0} and resizing to {1} on Y axis",
+								             data.Data.Y, data.Data.Height);
+							}
+						}
+						// Now we're sure this positions the note within the screen
 						window.Move (data.Data.X, data.Data.Y);
+					}
 
 					// This is here because emiting inside
 					// OnRealized causes segfaults.
@@ -1340,6 +1428,10 @@ namespace Tomboy
 						if (int.TryParse (xml.ReadString (), out num))
 							note.CursorPosition = num;
 						break;
+					case "selection-bound-position":
+						if (int.TryParse (xml.ReadString (), out num))
+							note.SelectionBoundPosition = num;
+						break;
 					case "width":
 						if (int.TryParse (xml.ReadString (), out num))
 							note.Width = num;
@@ -1475,6 +1567,10 @@ namespace Tomboy
 			xml.WriteStartElement (null, "cursor-position", null);
 			xml.WriteString (note.CursorPosition.ToString ());
 			xml.WriteEndElement ();
+			
+			xml.WriteStartElement (null, "selection-bound-position", null);
+			xml.WriteString (note.SelectionBoundPosition.ToString ());
+			xml.WriteEndElement ();
 
 			xml.WriteStartElement (null, "width", null);
 			xml.WriteString (note.Width.ToString ());
@@ -1574,9 +1670,10 @@ namespace Tomboy
 
 			if ((bool) Preferences.Get (Preferences.ENABLE_DELETE_CONFIRM)) {
 				// show confirmation dialog
-				if (notes.Count == 1)
-					message = Catalog.GetString ("Really delete this note?");
-				else
+				if (notes.Count == 1) {
+					Note note = notes[0];
+					message = string.Format (Catalog.GetString ("Really delete \"{0}\"?"), note.Title) ;	
+				} else
 					message = string.Format (Catalog.GetPluralString (
 						"Really delete this {0} note?",
 						"Really delete these {0} notes?",
